package _800_sqlDb.ue011_UserDBExampel.View;

import _800_sqlDb.ue011_UserDBExampel.Model.DataModel;
import _800_sqlDb.ue011_UserDBExampel.Model.User;
import _800_sqlDb.ue011_UserDBExampel.Model.UserList;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/****************************************************************************************************
 * Die Klasse EditUser2_Frame implementiert eine grafische Benutzeroberfläche (GUI) zur Verwaltung von Benutzerdaten.
 * Es handelt sich um ein JFrame-Fenster, das es ermöglicht, Benutzer in einer Liste auszuwählen und deren Details 
 * in einem separaten Panel zu bearbeiten.
 *
 * Hauptfunktionen:
 * - Anzeige von Benutzerlisten mit Filtermöglichkeit
 * - Auswahl eines Benutzers über eine Liste oder eine Dropdown-Auswahlbox
 * - Bearbeiten und Anzeigen der Benutzerdetails
 *
 * Autor: Prof. Gaisberger
 * Version: 1.0
 ****************************************************************************************************/

public class EditUser2_Frame extends JInternalFrame {
    private static final long serialVersionUID = 1L;

    // -------------------------------------------------------------------------------------
    // Instanzvariablen/Klassenvariablen für GUI-Komponenten und das Datenmodell
    private DataModel database;                      // Datenmodell für den Zugriff auf die Datenbank
    private JTextField txtSelectUser;                // Textfeld für die Eingabe eines Filtertextes
    private JComboBox<User> boxSelectUser;           // Dropdown-Box zur Auswahl eines Benutzers
    private JList<User> listSelectUser;              // Liste zur Anzeige von Benutzern
    private DefaultListModel<User> userListModel;    // Datenmodell für die Benutzerliste

    /**
     * Konstruktor der Klasse.
     * Initialisiert das Fenster, die Komponenten und lädt die Benutzerliste und die Dropdown-Auswahl.
     *
     * @param aDatabase Datenbankmodell für den Zugriff auf die Benutzerdaten.
     */
    public EditUser2_Frame(DataModel aDatabase) {
        super("Benutzerverwaltung", true, true,false,false);
        
        database = aDatabase;
        
        setSize(600, 400); 								// Fenstergröße festlegen
        //setTitle("User Management"); 					// Fenster-Titel festlegen
        //setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); // Schließen des Fensters definiert
        //setLocationRelativeTo(null); 	// Fenster in der Mitte des Bildschirms platzieren
        
        initializeComponents(); // GUI-Komponenten initialisieren
        loadUserDropBox("%");   // Benutzerdaten in die Dropdown-Auswahl laden
        loadUserList("%");      // Benutzerdaten in die Liste laden (kein spezieller Filter)
        setVisible(true);       // Fenster sichtbar machen
    }

    /**
     * Initialisiert alle GUI-Komponenten und fügt sie dem Hauptfenster hinzu.
     * Dazu gehören die Benutzerliste, die Dropdown-Auswahlbox und das Panel zur Bearbeitung der Benutzerdetails.
     */
    private void initializeComponents() {
        getContentPane().setLayout(new BorderLayout());

        // ---------------------------------------------------------------------------------
        // SplitPane zur Trennung der Benutzerliste und des Detailpanels
        JSplitPane splitPane = new JSplitPane();
        splitPane.setDividerLocation(100);
        getContentPane().add(splitPane, BorderLayout.CENTER);

        // ---------------------------------------------------------------------------------
        // Panel zum Bearbeiten der Userdaten
        User_EditPanel panelUserDetail = new User_EditPanel(database);
        splitPane.setRightComponent(panelUserDetail);        
        
        // ---------------------------------------------------------------------------------
        // Panel für die Benutzerfilterung und -auswahl
        JPanel panelSelectUser = new JPanel();
        panelSelectUser.setLayout(new GridLayout(0, 3, 0, 0));
        getContentPane().add(panelSelectUser, BorderLayout.NORTH);

	        // Textfeld für die Eingabe eines Filtertextes
	        txtSelectUser = new JTextField();
	        txtSelectUser.setColumns(15); // Breite des Textfelds festlegen
	        txtSelectUser.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
	                loadUserDropBox(txtSelectUser.getText()); // Dropdown-Auswahl aktualisieren
	                loadUserList(txtSelectUser.getText());   // Benutzerliste aktualisieren
	            }
	        });
	
	        // Dropdown-Box zur Auswahl eines Benutzers
	        boxSelectUser = new JComboBox<>();
	        boxSelectUser.addActionListener(new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
	                User selectedUser = (User) boxSelectUser.getSelectedItem();
	                if (selectedUser != null) {
	                    panelUserDetail.setUser(selectedUser); // Benutzerdetails im Panel anzeigen
	                }
	            }
	        });
	
	        panelSelectUser.add(boxSelectUser);
	        panelSelectUser.add(txtSelectUser);

	        
        // ---------------------------------------------------------------------------------
        // Panel für die Benutzerliste
        JPanel panelUserList = new JPanel();
        splitPane.setLeftComponent(panelUserList);

	        userListModel = new DefaultListModel<>(); // Datenmodell für die Benutzerliste
	        listSelectUser = new JList<>(userListModel); // Benutzerliste erstellen
	        listSelectUser.addListSelectionListener(new ListSelectionListener() {
	            public void valueChanged(ListSelectionEvent e) {
	                if (!e.getValueIsAdjusting()) {
	                    User selectedUser = listSelectUser.getSelectedValue();
	                    if (selectedUser != null) {
	                        panelUserDetail.setUser(selectedUser); // Benutzerdetails im Panel anzeigen
	                    }
	                }
	            }
	        });
	
	        panelUserList.setLayout(new BorderLayout(0, 0));
	        JScrollPane scrollPane = new JScrollPane(listSelectUser); // ScrollPane für die Benutzerliste
	        panelUserList.add(scrollPane);
	        

    }

    
    /** ----------------------------------------------------------------------------------------<br>
     * Lädt Benutzer in die Dropdown-Auswahl basierend auf dem angegebenen Filter.
     * 
     * @param aFilter Filtertext zur Einschränkung der Benutzerliste.
     */
    public void loadUserDropBox(String aFilter) {
        if (aFilter == null || aFilter.isEmpty())
            aFilter = "%"; // Standardfilter (alle Benutzer anzeigen)

        UserList users = database.tbl_User().selectByName(aFilter);

        boxSelectUser.removeAllItems(); // Alle vorhandenen Einträge entfernen
        do {
            User user = users.getUser();
            boxSelectUser.addItem(user); // Benutzer zur Dropdown-Auswahl hinzufügen
        } while (users.next());
    }

    
    /** ----------------------------------------------------------------------------------------<br>
     * Lädt Benutzer in die Liste basierend auf dem angegebenen Filter.
     * 
     * @param aFilter Filtertext zur Einschränkung der Benutzerliste.
     */
    public void loadUserList(String aFilter) {
        if (aFilter == null || aFilter.isEmpty())
            aFilter = "%"; // Standardfilter (alle Benutzer anzeigen)

        UserList users = database.tbl_User().selectByName(aFilter);

        userListModel.clear(); // Liste leeren

        do {
            User user = users.getUser();
            userListModel.addElement(user); // Benutzer zur Liste hinzufügen
        } while (users.next());
    }
}
