package _800_sqlDb.ue011_UserDBExampel.View;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import javax.swing.*;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;

import _800_sqlDb.ue011_UserDBExampel.Model.DataModel;
import _800_sqlDb.ue011_UserDBExampel.Model.User;
import _800_sqlDb.ue011_UserDBExampel.Model.UserList;


/****************************************************************************************************
 * Die Klasse EditUser3_Table stellt eine Benutzerverwaltungsoberfläche bereit, die auf JFrame basiert.
 * Diese Benutzeroberfläche bietet folgende Funktionen:
 * - Benutzer anzeigen und deren Daten in einer Tabelle bearbeiten.
 * - Änderungen automatisch speichern, sobald die Daten bearbeitet werden.
 * - Benutzerliste filtern, indem ein Filtertext eingegeben wird.
 * - Ausgewählte Benutzer aus der Tabelle und Datenbank löschen.
 *
 * Autor: Prof. Gaisberger
 * Version: 1.0
 ****************************************************************************************************/

// ----------------------------------------------------------------------------------------------------
public class EditUser3_Table extends JInternalFrame {
	private static final long serialVersionUID = 1L;
	
	// -------------------------------------------------------------------------------------
	// Instanzvariablen/Klassenvariablen
	private DataModel database;				    // Datenmodell für den Zugriff auf die Datenbank
    private DefaultTableModel tableModel;	    // Tabellenmodell zur Verwaltung der Tabellendaten
    private JTable table;					    // Tabelle zur Anzeige der Benutzerdaten   
    private JButton btnDelete; 					// Button zum Löschen eines ausgewählten Benutzers

    
    /** ----------------------------------------------------------------------------------------<br>
     * Konstruktor der Klasse.
     * Initialisiert das Fenster, setzt grundlegende Eigenschaften und lädt die GUI-Komponenten.
     * 
     * @param aDatabase Datenbankmodell für den Zugriff auf die Benutzerdaten.
     */
    public EditUser3_Table(DataModel aDatabase) {
        super("Benutzerverwaltung", true, true,false,false);
        
        database = aDatabase;

        setSize(600, 400); 								// Fenstergröße festlegen
        //setTitle("User Management"); 					// Fenster-Titel festlegen
        //setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); // Schließen des Fensters definiert
        //setLocationRelativeTo(null); 	// Fenster in der Mitte des Bildschirms platzieren

        initializeComponents(); 	// Komponenten initialisieren
        loadData("%"); 				// Alle Benutzerdaten laden (kein spezieller Filter)
        setVisible(true); 			// Fenster sichtbar machen
    }

    
    /** ----------------------------------------------------------------------------------------<br>
     * Initialisiert alle GUI-Komponenten und fügt sie dem Hauptfenster hinzu.
     */
    private void initializeComponents() {
        getContentPane().setLayout(new BorderLayout());

        // ---------------------------------------------------------------------------------
        // Tabelle initialisieren und Tabellenmodell definieren
        tableModel = new DefaultTableModel(new Object[]{"id", "Name", "Email"}, 0) {
            @Override
            public boolean isCellEditable(int row, int column) {
                return column != 0;  // Die "id"-Spalte ist nicht bearbeitbar
            }
        };

        // Listener hinzufügen, der Änderungen in der Tabelle erkennt und speichert
        tableModel.addTableModelListener(new TableModelListener() {
            @Override
            public void tableChanged(TableModelEvent e) {
                if (e.getType() == TableModelEvent.UPDATE) {
                    for (int row = e.getFirstRow(); row <= e.getLastRow(); row++) {
                        saveUser(row); // Änderungen in der Datenbank speichern
                    }
                }
            }
        });


        // ---------------------------------------------------------------------------------
        // ScrollPane zur Anzeige der Tabelle
        table = new JTable(tableModel); // Tabelle erstellen
        JScrollPane scrollPane = new JScrollPane(table);
        getContentPane().add(scrollPane, BorderLayout.CENTER);

        
        // ---------------------------------------------------------------------------------
        // Panel für Buttons am unteren Rand
        JPanel panelButtons = new JPanel();
        getContentPane().add(panelButtons, BorderLayout.SOUTH);
	
	        btnDelete = new JButton("Delete Selected");
	        btnDelete.addActionListener(e -> deleteData()); // Aktion für den Delete-Button
	        panelButtons.add(btnDelete);

	        
        // ---------------------------------------------------------------------------------
        // Panel für die Filtereingabe am oberen Rand
        JPanel panelSelectUser = new JPanel();
		panelSelectUser.setLayout(new GridLayout(0, 4, 0, 0));
        getContentPane().add(panelSelectUser, BorderLayout.NORTH);
	
	        JTextField txtSelectUser = new JTextField();
	        txtSelectUser.setColumns(15); // Breite des Textfeldes festlegen
	        txtSelectUser.addActionListener(e -> loadData(txtSelectUser.getText())); // Filter anwenden
	
	        JLabel lblNewLabel = new JLabel("Filter");
	        panelSelectUser.add(lblNewLabel);
	        panelSelectUser.add(txtSelectUser);
    }

    
    /** ----------------------------------------------------------------------------------------<br>
     * Lädt die Benutzerdaten aus der Datenbank basierend auf dem übergebenen Filter.
     * 
     * @param filter Filtertext zur Einschränkung der angezeigten Benutzerdaten.
     */
    private void loadData(String filter) {
        tableModel.setRowCount(0); // Tabelle zurücksetzen

        UserList userList = database.tbl_User().selectByName(filter); // Benutzerliste laden
        if (userList != null) {
            do {
                User user = userList.getUser();
                tableModel.addRow(new Object[]{user.getId(), user.getName(), user.getEmail()});
            } while (userList.next());
        }
    }

    
    /** ----------------------------------------------------------------------------------------<br>
     * Speichert die Änderungen eines bestimmten Benutzers in der angegebenen Tabellenzeile.
     * 
     * @param row Index der Tabellenzeile, die gespeichert werden soll.
     */
    private void saveUser(int row) {
        int id       = (int) tableModel.getValueAt(row, 0);
        String name  = (String) tableModel.getValueAt(row, 1);
        String email = (String) tableModel.getValueAt(row, 2);

        User user = new User(id, name, email);
        database.tbl_User().update(user); // Benutzerinformationen in der Datenbank aktualisieren
    }

    
    /** ----------------------------------------------------------------------------------------<br>
     * Speichert alle Benutzerdaten in der Tabelle.
     */
    private void saveAll() {
        for (int row = 0; row < tableModel.getRowCount(); row++) {
            saveUser(row);
        }
        JOptionPane.showMessageDialog(this, "Alle Datensätze gespeichert.");
    }

    
    /** ----------------------------------------------------------------------------------------<br>
     * Löscht den aktuell ausgewählten Benutzer aus der Tabelle und Datenbank.
     * Zeigt eine Nachricht an, wenn kein Benutzer ausgewählt wurde.
     */
    private void deleteData() {
        int selectedRow = table.getSelectedRow();
        if (selectedRow != -1)  {
            int id = (int) tableModel.getValueAt(selectedRow, 0);
            
            // database.tbl_User().deleteById(id); // Benutzer aus der Datenbank löschen 
            tableModel.removeRow(selectedRow); // Zeile aus der Tabelle entfernen
            
            JOptionPane.showMessageDialog(this, "User erfolgreich gelöscht!");
        } else {
            JOptionPane.showMessageDialog(this, "Kein User ausgewählt.");
        }
    }
}
