package _800_sqlDb.ue011_UserDBExampel.View;

import _800_sqlDb.ue011_UserDBExampel.Model.DataModel;
import _800_sqlDb.ue011_UserDBExampel.Model.User;

import javax.swing.*;
import java.awt.*;


/****************************************************************************************************
 * 
 * Die Klasse User_EditPanel bietet eine moderne Eingabemaske zur Bearbeitung von Benutzerdaten.
 * Sie stellt Textfelder zur Bearbeitung von Benutzerinformationen wie ID, Name und E-Mail bereit 
 * und ermöglicht das Speichern oder Zurücksetzen der Änderungen.
 *
 * Autor: Prof. Gaisberger
 * Version: 3.0
 * 
 ****************************************************************************************************/
public class User_EditPanel extends JPanel {
	private static final long serialVersionUID = 1L;
	
	// -------------------------------------------------------------------------------------
	// Instanzvariablen/Klassenvariablen    
    private DataModel database;          // Datenbankmodell zur Verwaltung der Benutzerdaten
    private User user;                   // Aktuell bearbeiteter Benutzer
    private JTextField idField;          // Textfeld zur Anzeige der Benutzer-ID (nur lesbar)
    private JTextField nameField;        // Textfeld zur Eingabe des Benutzernamens
    private JTextField emailField;       // Textfeld zur Eingabe der Benutzer-E-Mail

    
    /** ----------------------------------------------------------------------------------------<br>
     * Konstruktor für das User_EditPanel.
     * Initialisiert das Panel mit einem weißen Hintergrund und ruft die Methode initializeComponents auf,
     * um die GUI-Komponenten zu erstellen und zu positionieren.
     *
     * @param aDatabase Das Datenbankmodell, das zum Laden und Speichern von Benutzerdaten verwendet wird.
     */
    public User_EditPanel(DataModel aDatabase) {
        setBackground(Color.WHITE);      // Weißer Hintergrund für ein modernes, sauberes Design
        this.database = aDatabase;       // Datenbankmodell speichern
        
        initializeComponents();          // GUI-Komponenten initialisieren
        setVisible(true);                // Panel sichtbar machen
    }

    
    
    /** ----------------------------------------------------------------------------------------<br>
     * Methode zur Initialisierung der GUI-Komponenten.
     * Erstellt das Layout, fügt die Eingabefelder und Buttons hinzu und ordnet diese an.
     */
    private void initializeComponents() {
        setLayout(new BorderLayout(10, 10));  // BorderLayout mit Abständen zwischen den Komponenten
        
        //-------------------------------------------------------------------------------
        // Überschrift des Panels
        JLabel titleLabel = new JLabel("Benutzerdaten bearbeiten", SwingConstants.CENTER);
        titleLabel.setFont(new Font("Arial", Font.BOLD, 18));   // Schriftart und Größe setzen
        titleLabel.setForeground(new Color(50, 50, 50));        // Schriftfarbe auf Dunkelgrau setzen
        add(titleLabel, BorderLayout.NORTH);                   // Überschrift oben hinzufügen

        
        //-------------------------------------------------------------------------------
        // Panel für die Eingabefelder
        JPanel formPanel = new JPanel();
        GridBagLayout gbl_formPanel = new GridBagLayout();
        gbl_formPanel.columnWeights = new double[]{0.0, 1.0};  // Gewichte der Spalten setzen
        gbl_formPanel.rowHeights = new int[]{38, 0, 0};        // Höhen der Zeilen definieren
        gbl_formPanel.columnWidths = new int[]{56, 0};         // Breiten der Spalten definieren
        formPanel.setLayout(gbl_formPanel);
        formPanel.setBackground(Color.WHITE);                  // Hintergrundfarbe setzen

        add(formPanel, BorderLayout.CENTER);                   // Panel in die Mitte des Layouts setzen

        //-------------------------------------------------------------------------------
        // ID-Feld (nur lesbar)
        //-------------------------------------------------------------------------------
        JLabel label = new JLabel("ID:");
	        label.setFont(new Font("Arial", Font.PLAIN, 14));      // Schriftart und -größe setzen
	        GridBagConstraints gbc_label = new GridBagConstraints();
	        gbc_label.anchor = GridBagConstraints.EAST;            // Label rechtsbündig ausrichten
	        gbc_label.insets = new Insets(10, 10, 10, 10);         // Abstände setzen
	        gbc_label.gridx = 0;
	        gbc_label.gridy = 0;
	        formPanel.add(label, gbc_label);                       // Label zum Panel hinzufügen
	
        idField = new JTextField();
	        idField.setFont(new Font("Arial", Font.PLAIN, 14));    // Schriftart setzen
	        idField.setEditable(false);                            // Textfeld nur lesbar machen
	        idField.setColumns(20);                                // Breite des Textfelds
	        GridBagConstraints gbc = new GridBagConstraints();
	        gbc.insets = new Insets(10, 10, 10, 10);
	        gbc.fill = GridBagConstraints.HORIZONTAL;              // Textfeld horizontal ausfüllen lassen
	        gbc.gridx = 1;
	        gbc.gridy = 0;
	        formPanel.add(idField, gbc);                           // Textfeld zum Panel hinzufügen

        //-------------------------------------------------------------------------------
        // Name-Feld
        //-------------------------------------------------------------------------------
        label = new JLabel("Name:");
	        label.setFont(new Font("Arial", Font.PLAIN, 14));
	        gbc_label = new GridBagConstraints();
	        gbc_label.anchor = GridBagConstraints.EAST;
	        gbc_label.insets = new Insets(10, 10, 10, 10);
	        gbc_label.gridx = 0;
	        gbc_label.gridy = 1;
	        formPanel.add(label, gbc_label);
	
        nameField = new JTextField();
	        nameField.setFont(new Font("Arial", Font.PLAIN, 14));
	        nameField.setColumns(30);
	        gbc = new GridBagConstraints();
	        gbc.insets = new Insets(10, 10, 10, 10);
	        gbc.fill = GridBagConstraints.HORIZONTAL;
	        gbc.gridx = 1;
	        gbc.gridy = 1;
	        formPanel.add(nameField, gbc);

        //-------------------------------------------------------------------------------
        // E-Mail-Feld
        //-------------------------------------------------------------------------------
        label = new JLabel("E-Mail:");
	        label.setFont(new Font("Arial", Font.PLAIN, 14));
	        gbc_label = new GridBagConstraints();
	        gbc_label.anchor = GridBagConstraints.EAST;
	        gbc_label.insets = new Insets(10, 10, 10, 10);
	        gbc_label.gridx = 0;
	        gbc_label.gridy = 2;
	        formPanel.add(label, gbc_label);
	
        emailField = new JTextField();
	        emailField.setFont(new Font("Arial", Font.PLAIN, 14));
	        emailField.setColumns(30);
	        gbc = new GridBagConstraints();
	        gbc.insets = new Insets(10, 10, 10, 10);
	        gbc.fill = GridBagConstraints.HORIZONTAL;
	        gbc.gridx = 1;
	        gbc.gridy = 2;
	        formPanel.add(emailField, gbc);

        // Panel für die Buttons
        JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
        buttonPanel.setBackground(Color.WHITE);
        add(buttonPanel, BorderLayout.SOUTH);

	        JButton btnReset = new JButton("Zurücksetzen");
	        btnReset.addActionListener(e -> displayUser());
	        buttonPanel.add(btnReset);
	
	        JButton saveButton = new JButton("Speichern");
	        saveButton.addActionListener(e -> saveUser());
	        buttonPanel.add(saveButton);
    }

    
    
    /** ----------------------------------------------------------------------------------------<br>
     * Zeigt die aktuellen Benutzerdaten im Formular an.
     */
    private void displayUser() {
        if (user == null) return;

        idField.setText(String.valueOf(user.getId()));        // Benutzer-ID anzeigen
        nameField.setText(user.getName());                    // Benutzername anzeigen
        emailField.setText(user.getEmail());                  // Benutzer-E-Mail anzeigen
    }

    
    /** ----------------------------------------------------------------------------------------<br>
     * Speichert die Änderungen des aktuellen Benutzers in der Datenbank.
     */
    private void saveUser() {
        if (user == null) return;

        user.setName(nameField.getText());
        user.setEmail(emailField.getText());
        database.tbl_User().update(user);

        JOptionPane.showMessageDialog(this, "Benutzer erfolgreich aktualisiert.");
    }

    
    /** ----------------------------------------------------------------------------------------<br>
     * Setzt den aktuellen Benutzer und zeigt dessen Daten an.
     *
     * @param aUser Der Benutzer, der bearbeitet werden soll.
     */
    public void setUser(User aUser) {
        user = aUser;
        displayUser();
    }

    
    /** ----------------------------------------------------------------------------------------<br>
     * Setzt den aktuellen Benutzer anhand der Benutzer-ID und zeigt dessen Daten an.
     *
     * @param aId Die ID des Benutzers, der geladen werden soll.
     */
    public void setUser(int aId) {
        user = database.tbl_User().selectById(aId);
        displayUser();
    }

    // -------- Ende der Klasse --------
}
