package ue710_Fraktale;

import javax.swing.JPanel;

import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

/****************************************************************************************************
 * 
 * a Fractal will be Calcutet and Drawn here
 *
 * @author Sven Marceta
 * @version 1.0
 *
 ****************************************************************************************************
 */

//---------------------------------------------------------------------------------------------------
public class FraktelPanel extends JPanel
{
	// -------------------------------------------------------------------------------------
	// Instanzvariablen/Klassenvariablen
	private final int MAX_ITER = 500; // The maxium Iteration/Reseloution of the Mandelbrot
    private double ZOOM = 150; // incresing or decresing this Value will zoom in or zoom out of the Fractoul
    private final double  START_ZOOM   = 150; // The First ZOOM Level
    private BufferedImage I; // The Image Drawn of the MandlBrot
    private static double xChange = 0; //Value to move the x axise, Start on 0 normally
    private static double yChange = 0; //Value to move the y axise, strats on 0 normally
	private final double  INITIAL_MOVE_X = 0;  //Start Value of X
	private final double  INITIAL_MOVE_Y = 0;  //Start Value of y
	private double zoomValue = 1.5; // Value of wich it moves
	private int colorCalcValue = 16; // This value is used in the color Calculation *Best is from 13 to 20*
    
	// -------------------------------------------------------------------------------------
	// KONSTRUKTOR oder main
	/**
	 * Create the panel.
	 * @param I is an Image to draw the Mandelbrot
	 */
	public FraktelPanel()
	{
        setBounds(100, 100, 350,300);
        setLayout(null);
        //creating the imige
        I = new BufferedImage(this.getWidth(), this.getHeight(), BufferedImage.TYPE_INT_RGB);
        
        // Mouse listener for zooming in and out
     	addMouseListener(new MouseAdapter()
     	{
     		@Override
     		public void mouseClicked(MouseEvent e)
     		{
     			int x = e.getX();
     			int y = e.getY();

     			// changes the center to the postion of click
     			double newCenterX = (x - getWidth() / 2) / ZOOM + xChange;
     			double newCenterY = (y - getHeight() / 2) / ZOOM + yChange;

     			// Left-click zooms in and right-click zooms out
     			if (e.getButton() == MouseEvent.BUTTON1)
     			{
     				ZOOM *= zoomValue;
     			}
     			else if (e.getButton() == MouseEvent.BUTTON3)
     			{
     				ZOOM /= zoomValue;
     			}

     			// Goes to the center
     			xChange = newCenterX;
     			yChange = newCenterY;
     			FractulCalc();
     		}
     	});
        //System.out.printf("X:Hier \n"); //debbunging
     	FractulCalc();
    }

	// -------------------------------------------------------------------------------------
	// INSTANZMETHODEN
	/**
	 * Here the MandelBrot will be Calculated
	 *@param zx, zy, cX, cY, tmp, all are used to calcuten the Fractule
	 */
	public void FractulCalc()
	{
		double zx, zy, cX, cY, tmp; // All the valuese neede for the Calcoulation of the Mandelbrot
		
		for (int x = 0; x < this.getWidth(); x++) {
        	//System.out.printf("X: %d \n", 3); //debugging
            for (int y = 0; y < this.getHeight(); y++) {
            	
                zx = zy = 0;
                cX = (x - this.getWidth()/1.5) / ZOOM+ xChange; //Where the imigae is drawn
                cY = (y - this.getHeight()/2) / ZOOM+ yChange; //Where the imigae is drawn
                int iter = 0;
                while (zx * zx + zy * zy < 4 && iter < MAX_ITER) {
                     
                    tmp = zx;
                    zx = zx * zx - zy * zy + cX; //xChange chanchen th postiotn of the Drawing
                    zy = 2.0 * zy * tmp + cY; //xChange chanchen th postiotn of the Drawing
                    iter++;
                }
                //System.out.printf("X: %d \n", x); //debbunging
                //System.out.printf("With %d \n", this.getWidth()); //Debugging
                I.setRGB(x, y, iter | (iter << colorCalcValue));
            }
        }
        repaint(); // Start the paint Methode
	}
	
	/**
	 * Draw the BufferdImage ono the JPanel
	 * @param g, ist for the Graphics methode of Drawimiges
	 */
	public void paint(Graphics g)
    {
    	super.paint(g);
    	
    	// Creates the fractal image if it doesen't exist ord Value has changed
    	if (I == null || I.getWidth() != getWidth() || I.getHeight() != getHeight())
    	{
    		I = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_RGB);
    		FractulCalc();
    	}
    	
    	g.drawImage(I, 0, 0, this);
    }

	/**
	 * Zooms into the Imige.
	 */
	public void zoomIn()
	{
		ZOOM *= zoomValue;
		FractulCalc();
	}

	/**
	 * Zooms out of the Imige
	 */
	public void zoomOut()
	{
		ZOOM /= zoomValue;
		
		//debbuging
		System.out.printf("xCh: %.2f \n", ZOOM);
		
		FractulCalc();
	}
	
	public void move(double valueMovedX, double valueMovedY)
	{
		xChange += valueMovedX *ZOOM;
		yChange += valueMovedY *ZOOM;
		
		//Debbuging
		System.out.printf("xCh: %.2f \n", xChange);
		System.out.printf("yCh: %.2f \n", yChange);
		
		FractulCalc();
	}
	
	/**
	 * Resets the fractal to its initial zoom and position.
	 */
	public void reset()
	{
		ZOOM  = START_ZOOM;
		xChange = INITIAL_MOVE_X;
		yChange = INITIAL_MOVE_Y;
		FractulCalc();
	}
	
	/**
	 * Saves the current fractal image to a file, with a file chooser to select where ist is saved.
	 */
	public void save()
	{
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setDialogTitle("Save Fractal");
		fileChooser.setSelectedFile(new File("fractal.png")); // Default filename
		int userSelection = fileChooser.showSaveDialog(this);
		if (userSelection == JFileChooser.APPROVE_OPTION)
		{
			File fileToSave = fileChooser.getSelectedFile();
			// makes file to ong .png
			if (!fileToSave.getName().toLowerCase().endsWith(".png"))
			{
				fileToSave = new File(fileToSave.getAbsolutePath() + ".png");
			}

			// a try and catsch in oder to Svaqe the file
			try
			{
				ImageIO.write(I, "png", fileToSave);
				JOptionPane.showMessageDialog(this, "Image saved: " + fileToSave.getAbsolutePath());
			} catch (IOException e)
			{
				JOptionPane.showMessageDialog(this, "Error saving image: " + e.getMessage());
			}
		}
	}

	// -------- Ende der Klasse 
}
