package pUe391_ADObjekte;

import static org.junit.jupiter.api.Assertions.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;


/****************************************************************************************************
 * 
 * JUnit Test zum Prüfen der Aufgabe AD-Objects
 *
 * @author Prof. Gaisberger
 * @version 1.0
 *
 ****************************************************************************************************
 */

//@TestMethodOrder(MethodOrderer.OrderAnnotation.Alphanumeric.class)
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
//---------------------------------------------------------------------------------------------------
class _JUT_AD_Object
{
	// -------------------------------------------------------------------------------------
	// INSTANZVARIABLEN

	@BeforeEach
	void setUp() throws Exception
	{
		  // Hier nicht benötigt
	}

	
	@Test
	@Order(10)
	void testDomain()
	{
		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertDoesNotThrow(()->{new Domain("bsh.local","bsh");} );
		
		// NULL Pointer exceptions
		assertThrows(NullPointerException.class, ()->{new Domain(null,"bsh");} );
		assertThrows(NullPointerException.class, ()->{new Domain("bsh.local",null);} );

		// IllegalArguments
		assertThrows(IllegalArgumentException.class, ()->{new Domain("","bsh");} );
		assertThrows(IllegalArgumentException.class, ()->{new Domain("bsh.local","");} );

		// ------------------------------------------------------------------
		Domain dom = new Domain("bsh.local","bsh");
		assertEquals("bsh.local", dom.getName());
		assertEquals("bsh", dom.getBeschreibung());

		dom.setTopLevelDomain(true);
		assertTrue(dom.isTopLevelDomain());
		dom.setTopLevelDomain(false);
		assertFalse(dom.isTopLevelDomain());

		System.out.printf("%s \n", dom.toString());
	}

	@Test
	@Order(20)
	void testOrganisationalUnit()
	{
		// ------------------------------------------------------------------
		Domain dom = new Domain("bsh.local","bsh");

		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertDoesNotThrow(()->{new OrganisationalUnit("IT","EDV",dom);} );
		
		// NULL Pointer exceptions
		assertThrows(NullPointerException.class, ()->{new OrganisationalUnit(null,"EDV",dom);} );
		assertThrows(NullPointerException.class, ()->{new OrganisationalUnit("IT",null,dom);} );
		assertThrows(NullPointerException.class, ()->{new OrganisationalUnit("IT","EDV",null);} );

		// IllegalArguments
		assertThrows(IllegalArgumentException.class, ()->{new OrganisationalUnit("","EDV",dom);} );
		assertThrows(IllegalArgumentException.class, ()->{new OrganisationalUnit("IT","",dom);} );

		
		// ------------------------------------------------------------------
		OrganisationalUnit ou = new OrganisationalUnit("IT","EDV",dom);
		assertEquals(dom, ou.getDomain());
		assertEquals("IT", ou.getName());
		assertEquals("EDV", ou.getBeschreibung());

		ou.setDeleteAble(true);
		assertTrue(ou.isDeleteAble());
		ou.setDeleteAble(false);
		assertFalse(ou.isDeleteAble());
		
		System.out.printf("%s \n", ou.toString());
	}

	
	@Test
	@Order(30)
	void testBenutzerGruppe()
	{
		// ------------------------------------------------------------------
		Domain dom = new Domain("bsh.local","bsh");

		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertDoesNotThrow(()->{new BenutzerGruppe("admin","IT-Admins",dom);} );
		
		// NULL Pointer exceptions
		assertThrows(NullPointerException.class, ()->{new BenutzerGruppe(null,"IT-Admins",dom);} );
		assertThrows(NullPointerException.class, ()->{new BenutzerGruppe("admin",null,dom);} );
		assertThrows(NullPointerException.class, ()->{new BenutzerGruppe("admin","IT-Admins",null);} );

		// IllegalArguments
		assertThrows(IllegalArgumentException.class, ()->{new BenutzerGruppe("","IT-Admins",dom);} );
		assertThrows(IllegalArgumentException.class, ()->{new BenutzerGruppe("admin","",dom);} );

		
		// ------------------------------------------------------------------
		BenutzerGruppe grp = new BenutzerGruppe("admin","IT-Admins",dom);
		assertEquals(dom, grp.getDomain());
		assertEquals("admin", grp.getName());
		assertEquals("IT-Admins", grp.getBeschreibung());
		
		System.out.printf("%s \n", grp.toString());
	}

	
	@Test
	@Order(40)
	void testBenutzer()
	{
		// ------------------------------------------------------------------
		Domain dom = new Domain("bsh.local","bsh");
		BenutzerGruppe grp = new BenutzerGruppe("admin","IT-Admins",dom);

		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertDoesNotThrow(()->{new Benutzer("su","Admin",grp);} );
		
		// NULL Pointer exceptions
		assertThrows(NullPointerException.class, ()->{new Benutzer(null,"Admin",grp);} );
		assertThrows(NullPointerException.class, ()->{new Benutzer("su",null,grp);} );
		assertThrows(NullPointerException.class, ()->{new Benutzer("su","Admin",null);} );

		// IllegalArguments
		assertThrows(IllegalArgumentException.class, ()->{new Benutzer("","Admin",grp);} );
		assertThrows(IllegalArgumentException.class, ()->{new Benutzer("su","",grp);} );

		// ------------------------------------------------------------------
		Benutzer su = new Benutzer("su","Admin",grp);

		assertEquals("su", su.getName());
		assertEquals("Admin", su.getBeschreibung());
		
		System.out.printf("%s \n", su.toString());
	}

	@Test
	@Order(50)
	void testEMail()
	{
		// ------------------------------------------------------------------
		Domain dom = new Domain("bsh.local","bsh");
		BenutzerGruppe grp = new BenutzerGruppe("admin","IT-Admins",dom);
		Benutzer su = new Benutzer("su","Admin",grp);

		assertDoesNotThrow(()->{su.setEMail("su@bsh.at");} );
		assertEquals("su@bsh.at", su.getEMail());

		assertTrue(su.isValidEMail("su@bsh.at"));
		assertFalse(su.isValidEMail("sush.at"));
		
		assertThrows(NullPointerException.class, ()->{su.setEMail(null);} );
		assertThrows(IllegalArgumentException.class, ()->{su.setEMail("");} );
		assertThrows(IllegalArgumentException.class, ()->{su.setEMail("subsh.at");} );
	}

	@Test
	@Order(60)
	void testPasswort()
	{
		// ------------------------------------------------------------------
		Domain dom = new Domain("bsh.local","bsh");
		BenutzerGruppe grp = new BenutzerGruppe("admin","IT-Admins",dom);
		Benutzer su = new Benutzer("su","Admin",grp);

		assertDoesNotThrow(()->{su.setPasswort("Gk%12345");} );

		assertTrue(su.checkPasswort("Gk%12345"));
		assertFalse(su.checkPasswort("gk%1234"));
		
		assertTrue(su.isValidPasswort("GrKl1234$"));
		assertFalse(su.isValidPasswort("ungültig"));
		
		assertThrows(NullPointerException.class, ()->{su.setPasswort(null);} );
		assertThrows(IllegalArgumentException.class, ()->{su.setPasswort("");} );
		assertThrows(IllegalArgumentException.class, ()->{su.setPasswort("nix");} );

		assertThrows(NullPointerException.class, ()->{su.checkPasswort(null);} );
	}

	@Test
	@Order(70)
	void testInterface()
	{
		// ------------------------------------------------------------------
		Domain dom = new Domain("bsh.local","bsh");
		BenutzerGruppe grp = new BenutzerGruppe("admin","IT-Admins",dom);
		Benutzer su = new Benutzer("su","Admin",grp);

		su.setPasswort("Gk%12345");
		
		_ILogin login = (_ILogin) su;

		assertTrue(login.checkPasswort("Gk%12345"));
		assertFalse(login.checkPasswort("gk%1234"));

		assertThrows(NullPointerException.class, ()->{login.checkPasswort(null);} );
	}
	
	
	// -------- Ende der Klasse 
}
