package ue141L_HerrDerRinge;
import static org.junit.jupiter.api.Assertions.*;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;

/****************************************************************************************************
 * 
 * @author Prof. Gaisberger
 * @version 1.0
 *
 ****************************************************************************************************
 */

@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
//---------------------------------------------------------------------------------------------------
class _JUT_Wesen
{
	private static final String errNewNoError = "NEW: Mit diesen Parametern darf keine Exception ausgelöst werden!";
	private static final String errNewNullPointer  = "NEW: Parameter darf nicht NULL sein (EXCEPTION fehlt): ";;
	private static final String errNewEmptyStr = "NEW: Parameter darf kein Leerstring sein (EXCEPTION fehlt): ";
	private static final String errArgIllegal = "Ungültiger Wert! (EXCEPTION fehlt): ";
//	private static final String errArgNoError = "Dieser Wert ist gültig! Hier darf KEINE Exception ausgelöst werden: ";
	private static final String errReturnValue = "Der zurückgegeben Wert ist ungültig! Methode: ";
//	private static final String errReturnListe = "Die zurückgegebene Liste ist noch nicht korrekt! Methode: ";
	private static final String errNoException = "Hier darf keine Exception ausgelöst werden!";
	
	// -------------------------------------------------------------------------------------
	@BeforeEach
	void setUp() throws Exception
	{
		  // Hier nicht benötigt
	}
	
	// -------------------------------------------------------------------------------------
	@Test
	@Order(10)
	void testElfe()
	{
		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertThrows(NullPointerException.class, () -> {
			new Elfe(null, 70,"a", 150);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Elfe("", 70,"a", 150);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Elfe("Legolas", 0,"a", 150);
		}, errArgIllegal + "Stärke > 0");
		assertThrows(IllegalArgumentException.class, () -> {
			new Elfe("Legolas", 70,"a", -1);
		}, errArgIllegal + "Alter >= 0");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Elfe("Legolas", 70,"a", 150);
		}, errNewNoError);
		

		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Elfe elfe = new Elfe("Legolas", 70,"a", 150);
		
		assertEquals("Legolas", elfe.getName(),		errReturnValue + "getName");
		assertEquals(150, 		elfe.getAlter(),	errReturnValue + "getAlter");
		assertEquals(70,  		elfe.getStaerke(),	errReturnValue + "getStaerke");

		// --------------------------------------------------------------------------------
		assertThrows(IllegalArgumentException.class, () -> {
			elfe.setAlter(-1);
		}, errArgIllegal + "Alter >= 0");

		elfe.setAlter(125);
		assertEquals(125,	elfe.getAlter(),	errReturnValue + "getAlter");

		
		// --------------------------------------------------------------------------------
		assertThrows(IllegalArgumentException.class, () -> {
			new Elfe("Legolas", 0,"a", 150);
		}, errArgIllegal + "Stärke > 0");

		elfe.setStaerke(225);
		assertEquals(225,	elfe.getStaerke(),	errReturnValue + "getStaerke");
		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			elfe.setFreundVon(null);
		}, errArgIllegal + "setFreundVon: NULL erlaubt");

		Elfe freund = new Elfe("Freund",80,"a",150);
		elfe.setFreundVon(freund);
		assertEquals(freund, elfe.getFreundVon(), errReturnValue + "getFreundVon");
		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			elfe.bogenSchiessen();
			elfe.kaempfen();
			elfe.schlafen();
			elfe.laufen();
			elfe.paint(500, 100);
		}, errNoException );
	}

	// -------------------------------------------------------------------------------------
	@Test
	@Order(20)
	void testHobbit()
	{
		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertThrows(NullPointerException.class, () -> {
			new Hobbit(null, 70, "a", 150);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Hobbit("", 70,"a", 150);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Hobbit("Frodo", 0,"a", 150);
		}, errArgIllegal + "Stärke > 0");
		assertThrows(IllegalArgumentException.class, () -> {
			new Hobbit("Frodo", 70,"a", -1);
		}, errArgIllegal + "Alter >= 0");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Hobbit("Frodo", 70,"a", 150);
		}, errNewNoError);
		

		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Hobbit hobbit = new Hobbit("Frodo", 70,"a", 150);
		
		assertEquals("Frodo", 	hobbit.getName(),		errReturnValue + "getName");
		assertEquals(150, 		hobbit.getAlter(),		errReturnValue + "getAlter");
		assertEquals(70,  		hobbit.getStaerke(),	errReturnValue + "getStaerke");

		// --------------------------------------------------------------------------------
		assertThrows(IllegalArgumentException.class, () -> {
			hobbit.setAlter(-1);
		}, errArgIllegal + "Alter >= 0");

		hobbit.setAlter(125);
		assertEquals(125,	hobbit.getAlter(),	errReturnValue + "getAlter");

		
		// --------------------------------------------------------------------------------
		assertThrows(IllegalArgumentException.class, () -> {
			new Hobbit("Frodo", 0,"a", 150);
		}, errArgIllegal + "Stärke > 0");

		hobbit.setStaerke(225);
		assertEquals(225,	hobbit.getStaerke(),	errReturnValue + "getStaerke");
		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			hobbit.setFreundVon(null);
		}, errArgIllegal + "setFreundVon: NULL erlaubt");

		Hobbit freund = new Hobbit("Freund",80,"a",150);
		hobbit.setFreundVon(freund);
		assertEquals(freund, hobbit.getFreundVon(), errReturnValue + "getFreundVon");
		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			hobbit.verstecken();
			hobbit.kaempfen();
			hobbit.schlafen();
			hobbit.laufen();
			hobbit.paint(500, 100);
		}, errNoException );
	}


	// -------------------------------------------------------------------------------------
	@Test
	@Order(30)
	void testZauberer()
	{
		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertThrows(NullPointerException.class, () -> {
			new Zauberer(null, 70,"a", "Der Graue");
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Zauberer("", 70,"a", "Der Graue");
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Zauberer("Gandalf", 0,"a", "Der Graue");
		}, errArgIllegal + "Stärke > 0");
		
		assertThrows(NullPointerException.class, () -> {
			new Zauberer("Gandalf", 70,"a", null);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Zauberer("Gandalf", 70,"a", "");
		}, errNewEmptyStr);

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Zauberer("Gandalf", 70,"a", "Der Graue");
		}, errNewNoError);
		

		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Zauberer zauberer = new Zauberer("Gandalf", 70,"a", "Der Graue");
		
		assertEquals("Gandalf",  zauberer.getName(),	errReturnValue + "getName");
		assertEquals("Der Graue",zauberer.getStatus(),	errReturnValue + "getStatus");
		assertEquals(70,  		 zauberer.getStaerke(),	errReturnValue + "getStaerke");

		// --------------------------------------------------------------------------------
		assertThrows(NullPointerException.class, () -> {
			zauberer.setStatus(null);
		}, errArgIllegal + "setStatus");
		assertThrows(IllegalArgumentException.class, () -> {
			zauberer.setStatus("");
		}, errArgIllegal + "setStatus");

		zauberer.setStatus("Der Weisse");
		assertEquals("Der Weisse",	zauberer.getStatus(),	errReturnValue + "getStatus");

		
		// --------------------------------------------------------------------------------
		assertThrows(IllegalArgumentException.class, () -> {
			new Zauberer("Gandalf", 0,"a", "Der Graue");
		}, errArgIllegal + "Stärke > 0");

		zauberer.setStaerke(225);
		assertEquals(225,	zauberer.getStaerke(),	errReturnValue + "getStaerke");
		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			zauberer.setFreundVon(null);
		}, errArgIllegal + "setFreundVon: NULL erlaubt");

		Zauberer freund = new Zauberer("Freund",80,"a","Der Graue");
		zauberer.setFreundVon(freund);
		assertEquals(freund, zauberer.getFreundVon(), errReturnValue + "getFreundVon");
		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			zauberer.zaubern();
			zauberer.kaempfen();
			zauberer.schlafen();
			zauberer.laufen();
			zauberer.paint(300, 100);
		}, errNoException );
	}

	
	// -------------------------------------------------------------------------------------
	@Test
	@Order(40)
	void testZwerg()
	{
		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertThrows(NullPointerException.class, () -> {
			new Zwerg(null, 70,"a", 150);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Zwerg("", 70,"a", 150);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Zwerg("Gimli", 0,"a", 150);
		}, errArgIllegal + "Stärke > 0");
		assertThrows(IllegalArgumentException.class, () -> {
			new Zwerg("Gimli", 70,"a", 0);
		}, errArgIllegal + "Grösse > 0");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Zwerg("Gimli", 70,"a", 150);
		}, errNewNoError);
		

		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Zwerg zwerg = new Zwerg("Gimli", 70,"a", 150);
		
		assertEquals("Gimli", 	zwerg.getName(),		errReturnValue + "getName");
		assertEquals(150, 		zwerg.getGroesse(),		errReturnValue + "getGroesse");
		assertEquals(70,  		zwerg.getStaerke(),		errReturnValue + "getStaerke");

		// --------------------------------------------------------------------------------
		assertThrows(IllegalArgumentException.class, () -> {
			zwerg.setGroesse(0);
		}, errArgIllegal + "Grösse > 0");

		zwerg.setGroesse(125);
		assertEquals(125,	zwerg.getGroesse(),	errReturnValue + "getGroesse");

		
		// --------------------------------------------------------------------------------
		assertThrows(IllegalArgumentException.class, () -> {
			new Zwerg("Gimli", 0,"a", 150);
		}, errArgIllegal + "Stärke > 0");

		zwerg.setStaerke(225);
		assertEquals(225,	zwerg.getStaerke(),	errReturnValue + "getStaerke");
		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			zwerg.setFreundVon(null);
		}, errArgIllegal + "setFreundVon: NULL erlaubt");

		Zwerg freund = new Zwerg("Freund",80,"a",150);
		zwerg.setFreundVon(freund);
		assertEquals(freund, zwerg.getFreundVon(), errReturnValue + "getFreundVon");
		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			zwerg.streiten();
			zwerg.kaempfen();
			zwerg.schlafen();
			zwerg.laufen();
			zwerg.paint(400, 100);
		}, errNoException );
	}

	
	// -------------------------------------------------------------------------------------
	@Test
	@Order(50)
	void testOrk()
	{
		// ------------------------------------------------------------------------------
		// Teste Konstruktor
		assertThrows(NullPointerException.class, () -> {
			new Ork(null, 70,"a", "Anführer");
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Ork("", 70,"a", "Anführer");
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Ork("Uruk", 0,"a", "Anführer");
		}, errArgIllegal + "Stärke > 0");
		
		assertThrows(NullPointerException.class, () -> {
			new Ork("Uruk", 70,"a", null);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Ork("Uruk", 70,"a", "");
		}, errNewEmptyStr);

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Ork("Uruk", 70,"a", "Anführer");
		}, errNewNoError);
		

		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Ork ork = new Ork("Uruk", 70,"a", "Anführer");
		
		assertEquals("Uruk",  	ork.getName(),		errReturnValue + "getName");
		assertEquals("Anführer",ork.getRang(),		errReturnValue + "getRang");
		assertEquals(70,  		ork.getStaerke(),	errReturnValue + "getStaerke");

		// --------------------------------------------------------------------------------
		assertThrows(NullPointerException.class, () -> {
			ork.setRang(null);
		}, errArgIllegal + "setStatus");
		assertThrows(IllegalArgumentException.class, () -> {
			ork.setRang("");
		}, errArgIllegal + "setStatus");

		ork.setRang("Krieger");
		assertEquals("Krieger",	ork.getRang(),	errReturnValue + "getRang");

		
		// --------------------------------------------------------------------------------
		assertThrows(IllegalArgumentException.class, () -> {
			new Ork("Uruk", 0,"a", "Anführer");
		}, errArgIllegal + "Stärke > 0");

		ork.setStaerke(225);
		assertEquals(225,	ork.getStaerke(),	errReturnValue + "getStaerke");
		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			ork.setFreundVon(null);
		}, errArgIllegal + "setFreundVon: NULL erlaubt");

		Ork freund = new Ork("Freund",80,"a","Krieger");
		ork.setFreundVon(freund);
		assertEquals(freund, ork.getFreundVon(), errReturnValue + "getFreundVon");
		
		// --------------------------------------------------------------------------------
		assertDoesNotThrow(() -> {
			ork.streiten();
			ork.kaempfen();
			ork.schlafen();
			ork.laufen();
			ork.paint(500, 100);
		}, errNoException );
	}
	
	
	
	// -------- Ende der Klasse 
}
