package ue231_Zoo;
import static org.junit.jupiter.api.Assertions.*;
import org.junit.jupiter.api.MethodOrderer;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;

/****************************************************************************************************
 * 
 * JUnit Test für Wildtiere
 *
 * @author Prof. Gaisberger
 * @version 1.0
 *
 ****************************************************************************************************
 */

@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
//---------------------------------------------------------------------------------------------------
class _JUT_Wildtiere
{
	private static final String errNewNoError = "NEW: Mit diesen Parametern darf keine Exception ausgelöst werden!";
	private static final String errNewNullPointer  = "NEW: Parameter darf nicht NULL sein (EXCEPTION fehlt): ";;
	private static final String errNewEmptyStr = "NEW: Parameter darf kein Leerstring sein (EXCEPTION fehlt): ";
	private static final String errArgIllegal = "Ungültiger Wert! (EXCEPTION fehlt): ";
//	private static final String errArgNoError = "Dieser Wert ist gültig! Hier darf KEINE Exception ausgelöst werden: ";
	private static final String errReturnValue = "Der zurückgegeben Wert ist ungültig! Methode: ";

	// -----------------------------------------------------------------------------------
	@Test
	@Order(10)
	void testElefant()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new Elefant(null, "Wildtier", 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Elefant("", "Wildtier", 99);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Elefant("TIR999", null, 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Elefant("TIR999", "", 99);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Elefant("TIR999", "Wildtier", -1);
		}, errArgIllegal + "Alter");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Elefant("ELE999", "Wildtier", 99);
		}, errNewNoError);
		
		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Elefant neu = new Elefant("ELE999", "Wildtier", 99);
		
		assertEquals("ELE999",   neu.getId(),    errReturnValue + "getId");
		assertEquals("Wildtier", neu.getArt(),   errReturnValue + "getArt");
		assertEquals(99,         neu.getAlter(), errReturnValue + "getAlter");
		
		// --------------------------------------------------------------------------------
		neu.frissEtwas();
		neu.bewegeDich();
		neu.gibLaut();
		neu.waelzeDichImSchlamm();
	}

	// -----------------------------------------------------------------------------------
	@Test
	@Order(11)
	void testLoewe()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new Loewe(null, "Wildtier", 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Loewe("", "Wildtier", 99);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Loewe("TIR999", null, 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Loewe("TIR999", "", 99);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Loewe("TIR999", "Wildtier", -1);
		}, errArgIllegal + "Alter");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Loewe("LEO999", "Wildtier", 99);
		}, errNewNoError);
	
		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Loewe neu = new Loewe("LEO999", "Wildtier", 99);
		
		assertEquals("LEO999",   neu.getId(),    errReturnValue + "getId");
		assertEquals("Wildtier", neu.getArt(),   errReturnValue + "getArt");
		assertEquals(99,         neu.getAlter(), errReturnValue + "getAlter");
		
		// --------------------------------------------------------------------------------
		neu.frissEtwas();
		neu.bewegeDich();
		neu.gibLaut();
		neu.fangEineGazelle();
	}

	// -----------------------------------------------------------------------------------
	@Test
	@Order(12)
	void testGorilla()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new Gorilla(null, "Wildtier", 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Gorilla("", "Wildtier", 99);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Gorilla("TIR999", null, 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Gorilla("TIR999", "", 99);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Gorilla("TIR999", "Wildtier", -1);
		}, errArgIllegal + "Alter");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Gorilla("GOR999", "Wildtier", 99);
		}, errNewNoError);
	
		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Gorilla neu = new Gorilla("GOR999", "Wildtier", 99);
		
		assertEquals("GOR999",   neu.getId(),    errReturnValue + "getId");
		assertEquals("Wildtier", neu.getArt(),   errReturnValue + "getArt");
		assertEquals(99,         neu.getAlter(), errReturnValue + "getAlter");
		
		// --------------------------------------------------------------------------------
		neu.frissEtwas();
		neu.bewegeDich();
		neu.gibLaut();
		neu.klettereAufDenBaum();
	}

	// -----------------------------------------------------------------------------------
	@Test
	@Order(13)
	void testGazelle()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new Gazelle(null, "Wildtier", 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Gazelle("", "Wildtier", 99);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Gazelle("TIR999", null, 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Gazelle("TIR999", "", 99);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Gazelle("TIR999", "Wildtier", -1);
		}, errArgIllegal + "Alter");
		
		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Gazelle("GAZ999", "Wildtier", 99);
		}, errNewNoError);

		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Gazelle neu = new Gazelle("GAZ999", "Wildtier", 99);

		assertEquals("GAZ999",   neu.getId(),    errReturnValue + "getId");
		assertEquals("Wildtier", neu.getArt(),   errReturnValue + "getArt");
		assertEquals(99,         neu.getAlter(), errReturnValue + "getAlter");

		// --------------------------------------------------------------------------------
		neu.frissEtwas();
		neu.bewegeDich();
		neu.gibLaut();
		neu.fluechte();
	}

	// -----------------------------------------------------------------------------------
	@Test
	@Order(14)
	void testGepard()
	{
		// --------------------------------------------------------------------------------
		// Test new mit Null und Leerstrings!!!!
		assertThrows(NullPointerException.class, () -> {
			new Gepard(null, "Wildtier", 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Gepard("", "Wildtier", 99);
		}, errNewEmptyStr);

		assertThrows(NullPointerException.class, () -> {
			new Gepard("TIR999", null, 99);
		}, errNewNullPointer);
		assertThrows(IllegalArgumentException.class, () -> {
			new Gepard("TIR999", "", 99);
		}, errNewEmptyStr);

		assertThrows(IllegalArgumentException.class, () -> {
			new Gepard("TIR999", "Wildtier", -1);
		}, errArgIllegal + "Alter!");

		// Jetzt sollte alles in Ordnung sein!
		assertDoesNotThrow(() -> {
			new Gepard("GEP999", "Wildtier", 99);
		}, errNewNoError);

		// --------------------------------------------------------------------------------
		// new funktioniert jetzt
		Gepard neu = new Gepard("GEP999", "Wildtier", 99);

		assertEquals("GEP999",   neu.getId(),    errReturnValue + "getId");
		assertEquals("Wildtier", neu.getArt(),   errReturnValue + "getArt");
		assertEquals(99,         neu.getAlter(), errReturnValue + "getAlter");
		
		// --------------------------------------------------------------------------------
		neu.frissEtwas();
		neu.bewegeDich();
		neu.gibLaut();
		neu.fangEineGazelle();
	}
}
